# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pytest_jinja']

package_data = \
{'': ['*'], 'pytest_jinja': ['templates/bootstrap/*', 'templates/default/*']}

install_requires = \
['jinja2', 'pytest-metadata', 'pytest>=6.2.5,<7.0.0']

entry_points = \
{'pytest11': ['pytest-jinja = pytest_jinja.pytest_jinja']}

setup_kwargs = {
    'name': 'pytest-jinja',
    'version': '0.1.3',
    'description': 'A plugin to generate customizable jinja-based HTML reports in pytest',
    'long_description': '===================\npytest-jinja\n===================\n\npytest-jinja is a plugin to generate customizable jinja-based HTML reports in pytest.\nIt\'s based on pytest-html, but changes its inner working completely by separating the results data collection and the report generation, allowing easy developent of custom HTML reports that can include any javascript or css.\n\n\n.. image:: https://img.shields.io/badge/license-MPL%202.0-blue.svg\n   :target: https://github.com/magmax/pytest-jinja/blob/master/LICENSE\n   :alt: License\n\n.. image:: https://img.shields.io/pypi/pyversions/pytest-jinja.svg\n    :target: https://pypi.org/project/pytest-jinja\n    :alt: Python versions\n\n.. image:: https://img.shields.io/github/issues-raw/magmax/pytest-jinja.svg\n    :target: https://github.com/magmax/pytest-jinja/issues\n    :alt: Issues\n\nThis version is a fork from [g-bon\'s pytest-jinja](https://github.com/g-bon/pytest-jinja).\n\n----\n\n\nRequirements\n------------\n\nYou will need the following prerequisites in order to use pytest-html:\n\n* Python 3.10\n\n\nInstallation\n------------\n\nYou can install "pytest-jinja" via `pip`_ from `PyPI`_::\n\n    $ pip install pytest-jinja\n\nUsage\n-----\n- if no template is specified a default template is used. The default template looks almost identical to pytest-html::\n\n    $ pytest testcase --report=test_report.html\n\n- or you can pass your own template, pytest-jinja will render your template passing in the report data as jinja variables::\n\n    $ pytest testcase --report=test_report.html --template=my_template.html\n\nCreating a custom template\n--------------------------\nYou can create your own template by simply creating any template. The report data is "passed" to the page as a single object called `report`. The attributes of this object contain all the necessary report data.\n\nTemplate Example\n----------------\n.. code-block:: django\n\n    <html lang="en">\n    <head>\n        <meta charset="UTF-8">\n        <title>Test Report - {{ report.time_report_generation }}</title>\n    </head>\n    <body>\n    <h1>Test Report - {{ report.time_report_generation }} </h1>\n\n    <h2>Environment</h2>\n    <table id="environment">\n        {% for name,value in report.environment.items() %}\n        <tr>\n            <td>{{ name }}</td>\n            <td>{{ value }}</td>\n        </tr>\n        {% endfor %}\n    </table>\n\n    <h2>Summary</h2>\n    <p>{{ report.tests_count }} tests ran in {{ report.duration | round(2)}} seconds. </p>\n\n    <h2>Results</h2>\n    <table>\n        {% for r in report.results %}\n        <tr>\n            <td>{{ r.test_id }}</td>\n            <td>{{ r.outcome }}</td>\n            <td><strong>{{ r.time|round(5) }}s</strong></td>\n        </tr>\n        {% endfor %}\n    </table>\n\n    </body>\n    </html>\n\nAnother Template Example\n------------------------\n.. code-block::\n\n    {{ report | json }}\n\nAvailable Report Data\n---------------------\n\n`report.tests_count` : the total number of tests executed (int)\n\n`report.errors` : the number of errors (int)\n\n`report.failed` : the number of failed tests (int)\n\n`report.passed` : the number of passed tests (int)\n\n`report.skipped` : the number of skipped tests (int)\n\n`report.xfailed` : the number of expected failures (int)\n\n`report.xpassed`: the number of unexpected passes (int)\n\n`report.rerun`: the number of reruns (int)\n\n`report.duration` : the test session duration in seconds (float)\n\n`report.time_report_generation` : date and time of report generation (str)\n\n`report.environment`: metadata on tests execution (dict)\n\n`report.results`: the test results data (Object with attributes test_id, time, outcome, stacktrace, config)\n\n`report.report_path`: report path passed via command line (pathlib.Path)\n\n`report.template_path`: template path passed via command line (pathlib.Path)\n\n\n\nContributing\n------------\nContributions are very welcome. Tests can be run with `tox`_.\n\n\nLicense\n-------\nDistributed under Mozilla Public License, v. 2.0. If a copy of the MPL was not distributed with this\nfile, You can obtain one at http://mozilla.org/MPL/2.0/. "pytest-jinja" is free and open source software\n\n\nIssues\n------\n\nIf you encounter any problems, please `file an issue`_ along with a detailed description.\n\nThis `pytest`_ plugin was generated with `Cookiecutter`_ along with `@hackebrot`_\'s `cookiecutter-pytest-plugin`_ template.\n\n\n.. _`Cookiecutter`: https://github.com/audreyr/cookiecutter\n.. _`@hackebrot`: https://github.com/hackebrot\n.. _`cookiecutter-pytest-plugin`: https://github.com/pytest-dev/cookiecutter-pytest-plugin\n.. _`file an issue`: https://github.com/magmax/pytest-jinja/issues\n.. _`pytest`: https://github.com/pytest-dev/pytest\n.. _`tox`: https://tox.readthedocs.io/en/latest/\n.. _`pip`: https://pypi.org/project/pip/\n.. _`PyPI`: https://pypi.org/project\n',
    'author': 'Gabriele Bonetti',
    'author_email': 'gabriele.bonetti@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/g-bon/pytest-jinja',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import datetime
import os
import time
from pathlib import Path

import jinja2

OPTION_REPORT = "report"
OPTION_TEMPLATE = "template"


def pytest_addoption(parser):
    group = parser.getgroup("pytest-jinja")
    group.addoption(
        "--report",
        action="store",
        dest=OPTION_REPORT,
        default=None,
        help="The report destination.",
    )

    group.addoption(
        "--template",
        action="store",
        dest=OPTION_TEMPLATE,
        default=None,
        help="A jinja-based template.",
    )


def pytest_configure(config):
    report_path = config.getoption(OPTION_REPORT)
    template_path = config.getoption(OPTION_TEMPLATE)

    if report_path and not hasattr(config, "slaveinput"):
        config._report = JinjaReport(report_path, template_path, config)
        config.pluginmanager.register(config._report)


def pytest_unconfigure(config):
    report = getattr(config, "_report", None)
    if report:
        del config._report
        config.pluginmanager.unregister(report)


class JinjaReport:
    def __init__(self, report_path, template_path, config):
        self.report_path = Path(os.path.expandvars(report_path)).expanduser().resolve()
        self.template_path = (
            Path(template_path) if template_path else Path(__file__).parent / "templates/default/default.html"
        )
        self.results = []
        self.tests_count = 0
        self.errors = self.failed = 0
        self.passed = self.skipped = 0
        self.xfailed = self.xpassed = 0
        has_rerun = config.pluginmanager.hasplugin("rerunfailures")
        self.rerun = 0 if has_rerun else None
        self.config = config
        self.suite_start_time = None
        self.duration = None
        self.time_report_generation = None
        self.environment = None

    class TestResult:
        def __init__(self, report, config):
            self.test_id = report.nodeid
            if getattr(report, "when", "call") != "call":
                self.test_id = "::".join([report.nodeid, report.when])
            self.time = getattr(report, "duration", 0.0)
            self.outcome = report.outcome
            self.stacktrace = str(report.longrepr) if report.longrepr else report.longrepr
            self.config = config
            self.user_properties = report.user_properties

        def __lt__(self, other):
            order = ("Error", "Failed", "Rerun", "XFailed", "XPassed", "Skipped", "Passed")
            return order.index(self.outcome) < order.index(other.outcome)

    def count_passed(self, report, result):
        if report.when == "call":
            self.results.append(result)

            if hasattr(report, "wasxfail"):
                self.xpassed += 1
            else:
                self.passed += 1

    def count_failed(self, report, result):
        self.results.append(result)

        if getattr(report, "when", None) == "call":
            if hasattr(report, "wasxfail"):
                # pytest < 3.0 marked xpasses as failures
                self.xpassed += 1
            else:
                self.failed += 1
        else:
            self.errors += 1

    def count_skipped(self, report, result):
        self.results.append(result)

        if hasattr(report, "wasxfail"):
            self.xfailed += 1
        else:
            self.skipped += 1

    def count_other(self, report, result):
        # For now, the only "other" the plugin give support is rerun
        self.rerun += 1
        self.results.append(result)

    def _generate_report(self, session):
        suite_stop_time = time.time()
        self.duration = suite_stop_time - self.suite_start_time
        self.time_report_generation = datetime.datetime.now().strftime("%d/%m/%Y, %H:%M:%S")
        self.environment = self._generate_environment(session.config)
        self.tests_count = self.passed + self.failed + self.xpassed + self.xfailed

        templates_path = self.template_path.parent
        jinja_template_env = jinja2.Environment(
            loader=jinja2.FileSystemLoader(str(templates_path)),
            autoescape=jinja2.select_autoescape(["html", "xml"]),
        )

        template = jinja_template_env.get_template(str(Path(self.template_path).name))
        Path(self.report_path.parent).mkdir(parents=True, exist_ok=True)

        template.stream(
            report=self,
        ).dump(str(self.report_path))

    def _generate_environment(self, config):
        if not hasattr(config, "_metadata"):
            return dict()
        if config._metadata is None:
            return dict()
        return config._metadata

    def pytest_runtest_logreport(self, report):
        result = self.TestResult(report, self.config)

        if report.passed:
            self.count_passed(report, result)
        elif report.failed:
            self.count_failed(report, result)
        elif report.skipped:
            self.count_skipped(report, result)
        else:
            self.count_other(report, result)

    def pytest_collectreport(self, report):
        if report.failed:
            self.count_failed(report)

    def pytest_sessionstart(self, session):
        self.suite_start_time = time.time()

    def pytest_sessionfinish(self, session):
        self._generate_report(session)

    def pytest_terminal_summary(self, terminalreporter):
        terminalreporter.write_sep("-", f"generated file: {self.report_path}")
